import {createStore} from "vuex";
import {setLocalStorage, getLocalStorage} from "../utils";
import {Obj, StrNumObj} from "../interface";
import router from "../router"

interface State {
  userInfo: StrNumObj,
  chatUserInfo: Obj,
  isShowChat: boolean,
  isShowUserInfo: boolean,
  isShowGroupInfo: boolean,
  isShowRight: boolean,
  isSearching: boolean,
  userList: any[],
}

export default createStore({
  state: {
    userInfo: getLocalStorage('userInfo') || {}, //自己的信息
    chatUserInfo: {
      friendId: 0,
      friendName: '',
      id: 0,
      isGroup: 0,
      noReadNum: 0,
      userId: 0,
      userName: '',
      nickname: '',
    },  //当前聊天用户的信息
    isShowChat: false,
    isShowUserInfo: false,
    isShowGroupInfo: false,
    isShowRight: false,
    isSearching: false,
    userList: [],
  } as State,
  mutations: {
    logout(state) {
      state.userInfo.token = ''
      setLocalStorage('userInfo', state.userInfo)
      router.push('/')
    },
    setUserInfo(state, data) {
      state.userInfo = data
      setLocalStorage('userInfo', data)
    },
    updateUserInfo(state, obj) {
      state.userInfo = {...state.userInfo, ...obj}
      setLocalStorage('userInfo', state.userInfo)
    },
    setChatUserInfo(state, data) {
      state.chatUserInfo = data
    },
    setIsShowChat(state, status) {
      state.isShowChat = status
    },
    setIsShowUserInfo(state, status) {
      state.isShowUserInfo = status
    },
    setIsShowGroupInfo(state, status) {
      state.isShowGroupInfo = status
    },
    setIsSearching(state, status) {
      state.isSearching = status
    },
    setIsShowRight(state, status) {
      state.isShowRight = status
    },
    setNotReadMessage(state, id) {
      state.userList.forEach((item: any) => {
        if (id === item.friendId) {
          item.noReadNum++;
        }
      })
    },
  },
  actions: {},
  modules: {}
});
