import store from "../store";
import config from '../config'

const rawHeaderLen = 16;
const packetOffset = 0;
const headerOffset = 4;
const verOffset = 6;
const opOffset = 8;
const seqOffset = 12;

function Client(options: any) {
  let MAX_CONNECT_TIMES = 10;
  let DELAY = 15000;
  // @ts-ignore
  let self = this;
  self.connectStatus = 'connecting';
  self.options = options || {};
  self.createConnect(MAX_CONNECT_TIMES, DELAY);
}

Client.prototype.createConnect = function (max: number, delay: number) {
  let self = this;
  if (max === 0) {
    return;
  }
  connect();

  let textDecoder = new TextDecoder();
  let textEncoder = new TextEncoder();
  let heartbeatInterval: any;

  function connect() {
    let ws = new WebSocket(config.socketUrl);
    ws.binaryType = 'arraybuffer';
    ws.onopen = function () {
      auth();
    }

    ws.onmessage = function (evt) {
      let data = evt.data;
      let dataView = new DataView(data, 0);
      let packetLen = dataView.getInt32(packetOffset);
      let headerLen = dataView.getInt16(headerOffset);
      let ver = dataView.getInt16(verOffset);
      let op = dataView.getInt32(opOffset);
      let seq = dataView.getInt32(seqOffset);

      switch (op) {
        case 8:
          // auth reply ok
          self.connectStatus = 'connected'
          // send a heartbeat to server
          heartbeat();
          heartbeatInterval = setInterval(heartbeat, 30 * 1000);
          break;
        case 3:
          // receive a heartbeat from server
          break;
        case 9:
          // batch message
          for (let offset = rawHeaderLen; offset < data.byteLength; offset += packetLen) {
            // parse
            let packetLen = dataView.getInt32(offset);
            let headerLen = dataView.getInt16(offset + headerOffset);
            let ver = dataView.getInt16(offset + verOffset);
            let op = dataView.getInt32(offset + opOffset);
            let seq = dataView.getInt32(offset + seqOffset);
            let msgBody = textDecoder.decode(data.slice(offset + headerLen, offset + packetLen));
            // callback
            messageReceived(msgBody);
          }
          break;
        default:
          let msgBody = textDecoder.decode(data.slice(headerLen, packetLen));
          messageReceived(msgBody);
          break
      }
    }

    ws.onclose = function () {
      if (heartbeatInterval) clearInterval(heartbeatInterval);
      self.connectStatus = 'fail'
      setTimeout(reConnect, delay);
    }

    self.createNewConnect = function () {
      ws.close();
    }

    function heartbeat() {
      let headerBuf = new ArrayBuffer(rawHeaderLen);
      let headerView = new DataView(headerBuf, 0);
      headerView.setInt32(packetOffset, rawHeaderLen);
      headerView.setInt16(headerOffset, rawHeaderLen);
      headerView.setInt16(verOffset, 1);
      headerView.setInt32(opOffset, 2);
      headerView.setInt32(seqOffset, 1);
      ws.send(headerBuf);
    }

    function auth() {
      let token = `{"mid": ${store.state.userInfo.id}, "room_id":"live://1000", "platform":"web", "accepts":[1000, 1001, 1002]}`;
      let headerBuf = new ArrayBuffer(rawHeaderLen);
      let headerView = new DataView(headerBuf, 0);
      let bodyBuf = textEncoder.encode(token);
      headerView.setInt32(packetOffset, rawHeaderLen + bodyBuf.byteLength);
      headerView.setInt16(headerOffset, rawHeaderLen);
      headerView.setInt16(verOffset, 1);
      headerView.setInt32(opOffset, 7);
      headerView.setInt32(seqOffset, 1);
      ws.send(mergeArrayBuffer(headerBuf, bodyBuf));
    }

    function messageReceived(body: string) {
      let notify = self.options.notify;
      if (notify) notify(body);
    }

    function mergeArrayBuffer(ab1: any, ab2: any) {
      let u81 = new Uint8Array(ab1),
        u82 = new Uint8Array(ab2),
        res = new Uint8Array(ab1.byteLength + ab2.byteLength);
      res.set(u81, 0);
      res.set(u82, ab1.byteLength);
      return res.buffer;
    }
  }

  function reConnect() {
    self.createConnect(--max, delay * 2);
  }
}

export default Client
