import axios from 'axios'
import Config from '../config'
import {Obj} from "../interface"
import {Toast} from 'vant'
import 'vant/es/toast/style'
import {Dialog} from 'vant';
import 'vant/es/dialog/style'
import store from "../store";

const defaults: Obj = axios.defaults;
defaults.headers['Content-Type'] = 'application/x-www-form-urlencoded;charset=utf-8'

const service = axios.create({
  baseURL: Config.baseUrl,
  timeout: 30 * 1000,
  method: 'post',
})

service.interceptors.request.use((config: Obj): Obj => {
    if (store.state.userInfo.token) {
      config.params = {...config.params, token: store.state.userInfo.token}
    }
    config.data = JSON.stringify(config.data)
    return config
  }, (error: Obj) => {
    Toast(error.message)
    return Promise.reject(error)
  }
)

service.interceptors.response.use((response: Obj): Obj => {
    const data = response.data
    if (data.code === 0) {
      return data
    } else {
      !store.state.isSearching && Toast(data.message)
      return Promise.reject(data)
    }
  }, (error: Obj): Obj => {
    let {response, message} = error
    if (response && response.status === 403) {
      Dialog.alert({
        title: '温馨提示',
        message: '登录超时，请登录',
        confirmButtonText: '前往登录'
      }).then(() => {
        store.commit('logout')
      });
      return Promise.reject(error)
    }
    if (message === "Network Error") {
      message = "网络异常"
    } else if (message.includes("timeout")) {
      message = "请求超时，请稍后再试!"
    }
    Toast(message)
    return Promise.reject(error)
  }
)

export default service
